@@ -2,6 +2,10 @@ import pandas
 import h5py
 from sparsedat import wrappers as SDT_wrappers
 from sparsedat import Sparse_Data_Table
+import warnings
+import anndata
+from scipy import sparse
+import numpy
 
 
 def write_pandas_csv(data_frame, file_path):
@@ -33,12 +37,72 @@ def load_mtx(mtx_file_path, features_file_path, barcodes_file_path):
 
     return sdt
 
-
-def convert_h5_to_sdt(
-        h5_file_path,
-        SDT_file_path,
-        cells_as_rows=True
+def convert_cellranger_h5_to_anndata(
+    h5_file_path
 ):
+
+    h5_file = h5py.File(h5_file_path)
+
+    cellranger_version = 2
+
+    if "matrix" in h5_file:
+        cellranger_version = 3
+
+    matrix_name = None
+
+    if cellranger_version == 2:
+        for key, value in h5_file.items():
+            matrix_name = key
+            break
+    else:
+        matrix_name = "matrix"
+
+    data = h5_file[matrix_name]["data"][()]
+    indices = h5_file[matrix_name]["indices"][()]
+    indptr = h5_file[matrix_name]["indptr"][()]
+    shape = h5_file[matrix_name]["shape"][()]
+
+    max_value = numpy.max(data)
+
+    if max_value < 255:
+        dtype = numpy.uint8
+    if max_value < 65535:
+        dtype = numpy.uint16
+    else:
+        dtype = numpy.uint32
+
+    matrix = sparse.csc_matrix((data, indices, indptr), shape=shape).tocsr().transpose().astype(dtype)
+
+    if cellranger_version == 2:
+        gene_names = [x.decode("UTF-8")
+                      for x in list(h5_file[matrix_name]["gene_names"])]
+        gene_ids = [x.decode("UTF-8")
+                    for x in list(h5_file[matrix_name]["gene_ids"])]
+    else:
+        gene_names = [x.decode("UTF-8")
+                      for x in list(h5_file[matrix_name]["features"]["name"])]
+        gene_ids = [x.decode("UTF-8")
+                    for x in list(h5_file[matrix_name]["features"]["id"])]
+
+    barcodes = [x.decode("UTF-8") for x in h5_file[matrix_name]["barcodes"][()]]
+
+    barcodes_df = pandas.DataFrame(
+        index=barcodes,
+        dtype=numpy.object
+    )
+
+    genes_df = pandas.DataFrame(gene_names, index=gene_ids, columns=["Gene Name"], dtype=numpy.object)
+
+    adata = anndata.AnnData(X=matrix, var=genes_df, obs=barcodes_df)
+
+    return adata
+
+
+def load_cellranger_h5(
+    h5_file_path,
+    cells_as_rows=True
+):
+
     h5_file = h5py.File(h5_file_path)
 
     cellranger_version = 2
@@ -125,6 +189,31 @@ def convert_h5_to_sdt(
     if cells_as_rows:
         sdt.transpose()
 
+    return sdt
+
+
+def convert_h5_to_sdt(
+    h5_file_path,
+    SDT_file_path,
+    cells_as_rows=True
+):
+    warnings.warn("Function renamed to convert_cellranger_h5_to_sdt")
+
+    return convert_cellranger_h5_to_sdt(
+        h5_file_path,
+        SDT_file_path,
+        cells_as_rows
+    )
+
+
+def convert_cellranger_h5_to_sdt(
+        h5_file_path,
+        SDT_file_path,
+        cells_as_rows=True
+):
+
+    sdt = load_cellranger_h5(h5_file_path, cells_as_rows)
+
     sdt.save(SDT_file_path)
 
     return sdt
